
#import "DrawContentView.h"


@implementation DrawContentView

@synthesize contentType;

-(id)initWithFrame:(CGRect)frame {
    
    self = [super initWithFrame:frame];
    if (self) {
		
		//表示用のビュー
        makerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, frame.size.width, frame.size.height)];
		[self addSubview:makerView];
		makerView.backgroundColor = [UIColor clearColor];
		
		//描画用コンテキスト
		CGColorSpaceRef	colorSpace = CGColorSpaceCreateDeviceRGB();
		drawContext = CGBitmapContextCreate(
										   nil, 
										   frame.size.width, 
										   frame.size.height, 
										   8, 
										   4 * frame.size.width, 
										   colorSpace, 
										   kCGImageAlphaPremultipliedFirst);
		CGColorSpaceRelease(colorSpace);
		CGContextSetRGBFillColor(drawContext, 0, 0, 0, 0);
		CGContextFillRect(drawContext, CGRectMake(0,0,frame.size.width,frame.size.height));
		CGContextSetShouldAntialias(drawContext, FALSE);
		CGContextSetAllowsAntialiasing(drawContext, FALSE);
		CGContextSetLineCap(drawContext, kCGLineCapRound);
		[self setLineColorRed:0.0 Green:0.0 Blue:0.0 Alpha:1.0];	// 色
		[self setLineSize:PEN_DEF_SIZE];							// 線の太さ
		
		//ペン用の配列
		makerArray = [NSMutableArray array];
    }
    return self;
}


- (void)dealloc 
{
	if(drawContext)
		CGContextRelease(drawContext);
	[self clearHitTestBuff];
}



//========================================================================
//ペンサイズ
//========================================================================
-(void)setLineSize:(CGFloat)line 
{
	penSize = line;
	CGContextSetLineWidth(drawContext, penSize);
}

//========================================================================
//ペンカラー
//========================================================================
-(void)setLineColorRed:(CGFloat)red 
				 Green:(CGFloat)green 
				  Blue:(CGFloat)blue 
				 Alpha:(CGFloat)alpha 
{
	penColR = red;
	penColG = green;
	penColB = blue;
	penColA = alpha;
	CGContextSetRGBStrokeColor(drawContext, penColR, penColG, penColB, penColA);
	CGContextSetRGBFillColor(drawContext, penColR, penColG, penColB, penColA);
}




#pragma mark -

//========================================================================
//タッチダウン
//========================================================================
-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	
	UITouch*	touch = [touches anyObject];
	CGPoint	pt = [touch locationInView:self];
	startPt = pt;	//開始点保存
	
	//ペン
	if(contentType==ContentTypePen){
		//ポイントを保存
		NSValue*	val = [NSValue valueWithCGPoint:startPt];
		currentMakerPts = [NSMutableArray array];
		[currentMakerPts addObject:val];
		//描画設定を辞書に記録
		NSDictionary*   dic = [NSDictionary dictionaryWithObjectsAndKeys:
							   currentMakerPts, @"makerPts",
							   [NSNumber numberWithFloat:penSize], @"penSize",
							   [NSNumber numberWithFloat:penColR], @"penColR",
							   [NSNumber numberWithFloat:penColG], @"penColG",
							   [NSNumber numberWithFloat:penColB], @"penColB",
							   [NSNumber numberWithFloat:penColA], @"penColA",
							   nil];
		//描画設定を記録した辞書を配列に保存
		[makerArray addObject:dic];
	}
	//選択
	else if(contentType==ContentTypeSelect ){
		hitDownIndex = [self hitTestX:pt.x Y:pt.y];
		[self restoreMarker];
	}
}

//========================================================================
//タッチムーブ
//========================================================================
-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event 
{
	UITouch*	touch = [touches anyObject];
	CGPoint		pt = [touch locationInView:self];
	
	//ペン
	if(contentType==ContentTypePen){
		//ポイントを保存
		NSValue*	val = [NSValue valueWithCGPoint:pt];
		[currentMakerPts addObject:val];
		
		CGContextSaveGState(drawContext);	//保存
		{
			CGContextTranslateCTM(drawContext, 0, makerView.frame.size.height);
			CGContextScaleCTM(drawContext, 1, -1);
			CGContextBeginPath(drawContext);
			CGContextMoveToPoint(drawContext, startPt.x, startPt.y);
			CGContextAddLineToPoint(drawContext, pt.x, pt.y);
			CGContextStrokePath(drawContext);
		}
		CGContextRestoreGState(drawContext);	//復元
		CGImageRef	cgImage = CGBitmapContextCreateImage(drawContext);
		makerView.layer.contents = (__bridge id)cgImage;
		CGImageRelease(cgImage);
		startPt = pt;
	}
	//選択
	else if(contentType==ContentTypeSelect){
		hitDownIndex = [self hitTestX:pt.x Y:pt.y];
		[self restoreMarker];
	}
}

//========================================================================
//タッチエンド
//========================================================================
-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event 
{
	UITouch*	touch = [touches anyObject];
	CGPoint		pt = [touch locationInView:self];
	
	//ペン
	if(contentType==ContentTypePen){
	}
	//選択
	else if(contentType==ContentTypeSelect){
		hitDownIndex = [self hitTestX:pt.x Y:pt.y];
		if(hitDownIndex!=-1){
			//ペン削除処理
			[makerArray removeObjectAtIndex:hitDownIndex];
			hitDownIndex = -1;
			[self restoreMarker];
			[self clearHitTestBuff];
			[self makeHitTestBuff];
		}
	}
}


//========================================================================
//ペン作成
//========================================================================
-(void)restoreMarker 
{
	//バッファをクリア
	CGContextClearRect(drawContext, 
					   CGRectMake(0, 0, CGBitmapContextGetWidth(drawContext), 
								  CGBitmapContextGetHeight(drawContext)));
	//ペンを復元
	for (int i=0; i < [makerArray count]; i++) {
		NSDictionary*   dic = [makerArray objectAtIndex:i];
		float			line_size = [[dic objectForKey:@"penSize"] floatValue];
		float			r = [[dic objectForKey:@"penColR"] floatValue];
		float			g = [[dic objectForKey:@"penColG"] floatValue];
		float			b = [[dic objectForKey:@"penColB"] floatValue];
		float			a = [[dic objectForKey:@"penColA"] floatValue];
		NSArray*        pts = [dic objectForKey:@"makerPts"];
		CGContextSaveGState(drawContext);		//保存
		{
			CGContextTranslateCTM(drawContext, 0, makerView.frame.size.height);
			CGContextScaleCTM(drawContext, 1, -1);
			CGContextBeginPath(drawContext);
			CGContextSetLineWidth(drawContext, line_size);
			//描画色設定
			if(i==hitDownIndex)
				CGContextSetRGBStrokeColor(drawContext, r, g, b, 0.5);
			else
				CGContextSetRGBStrokeColor(drawContext, r, g, b, a);
			//描画パス設定
			NSValue*	val = [pts objectAtIndex:0];
			CGPoint	stPt = [val CGPointValue];
			CGContextMoveToPoint(drawContext, stPt.x, stPt.y);
			for (int ii=1; ii < [pts count]; ii++) {
				NSValue*        val = [pts objectAtIndex:ii];
				CGPoint         pt = [val CGPointValue];
				CGContextAddLineToPoint(drawContext, pt.x, pt.y);
			}
			CGContextStrokePath(drawContext);
		}
		CGContextRestoreGState(drawContext);	//復元
	}
	CGImageRef	cgImage = CGBitmapContextCreateImage(drawContext);
	makerView.layer.contents = (__bridge id)cgImage;
	CGImageRelease(cgImage);
}

//========================================================================
//ヒットテストバッファ作成
//========================================================================
-(void)makeHitTestBuff 
{
	[self clearHitTestBuff];
	
	int		w = CGBitmapContextGetWidth(drawContext);
	int		h = CGBitmapContextGetHeight(drawContext);
	//描画用コンテキスト
	CGColorSpaceRef	colorSpace = CGColorSpaceCreateDeviceRGB();
	CGContextRef	context = CGBitmapContextCreate(
													nil,
													w, 
													h, 
													8, 
													4 * w, 
													colorSpace, 
													kCGImageAlphaPremultipliedFirst);
	CGColorSpaceRelease(colorSpace);
	hitTestContext = context;
	
	for (int i=0; i < [makerArray count]; i++) {
		NSDictionary*   dic = [makerArray objectAtIndex:i];
		float	r,g,b,a=1.0;
		float			line_size = [[dic objectForKey:@"penSize"] floatValue];
		NSArray*        pts = [dic objectForKey:@"makerPts"];
		
		CGContextSaveGState(context);	//保存
		{
			CGContextTranslateCTM(context, 0, makerView.frame.size.height);
			CGContextScaleCTM(context, 1, -1);
			for (int ii=0; ii < [pts count]; ii++) {
				NSValue*        val = [pts objectAtIndex:ii];
				CGPoint         pt = [val CGPointValue];
				if(ii==0){
					startPt = pt;
				}
				else{
					CGContextBeginPath(context);
					CGContextMoveToPoint(context, startPt.x, startPt.y);
					CGContextAddLineToPoint(context, pt.x, pt.y);
					CGContextSetLineWidth(context, line_size);
					//ピクセル値をペンのインデックス値にする
					r = 1.0 * ((1.0/255.0)*i);
					g = r;
					b = r;
					CGContextSetRGBStrokeColor(context, r, g, b, a);
					CGContextStrokePath(context);
					startPt = pt;
				}
			}
		}
		CGContextRestoreGState(context);	//復元
	}
}

//========================================================================
//ペンバッファのヒットチェック、ヒットしたピクセルのIndex値を返す。なにもなければ-1を返す
//========================================================================
-(int)hitTestX:(int)x Y:(int)y 
{
	int				hitIndex = -1;
	if(hitTestContext){
		unsigned char*	p;
		unsigned char*	ptr;
		//ヒットテスト
		int		preRowDst = CGBitmapContextGetBytesPerRow(hitTestContext);
		p = CGBitmapContextGetData(hitTestContext);
		if( p ){
			ptr = p+(x*4)+(y*preRowDst);
			int		a = *(ptr+0);
			if(a!=0){
				hitIndex = *(ptr+1);
			}
		}
	}
	
	
	return hitIndex;
}



//========================================================================
//ペンバッファクリア
//========================================================================
-(void)clearHitTestBuff 
{
	if(hitTestContext){
		CGContextRelease(hitTestContext);
		hitTestContext = nil;
	}
}

//========================================================================
//クリア
//========================================================================
-(void)clear
{
	[makerArray removeAllObjects];
	[self restoreMarker];
	[self clearHitTestBuff];
	[self makeHitTestBuff];
}





#pragma mark - アクセッサメソッド

//========================================================================
//コンテンツタイプを設定
//========================================================================
-(void)setContentType:(ContentType)ty {
	
	if(contentType == ty)
		return;
	
	//選択モードだった場合
	if(contentType==ContentTypeSelect){
		[self clearHitTestBuff];//ペンバッファクリア
	}
	
	contentType = ty;
	
	//ペンモードに変更された場合
	if(contentType==ContentTypePen){
		
	}
	//選択モードに変更された場合
	else if(contentType==ContentTypeSelect){
		[self makeHitTestBuff];//ペンバッファ作成
	}
}



@end
